%  Copyright (C) 2002-2004 David Roundy
%
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program; see the file COPYING.  If not, write to
%  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%  Boston, MA 02110-1301, USA.

\begin{code}
module Darcs.Commands.List ( list, query ) where

import Darcs.Commands ( DarcsCommand(..),
                        CommandControl(Command_data, Hidden_command),
                        command_alias,
                      )
import Darcs.Commands.ListFiles ( list_files, list_manifest )
import Darcs.Commands.ListTags ( list_tags )
import Darcs.Repository ( amInRepository )

list_description :: String
list_description = "Query information which is stored by darcs."

list_help :: String
list_help =
 "Use the --help option with the subcommands to obtain help for\n"++
 "subcommands (for example, \"darcs list files --help\").\n"

list :: DarcsCommand
list = SuperCommand {command_name = "list",
                      command_help = list_help,
                      command_description = list_description,
                      command_prereq = amInRepository,
                      command_sub_commands = [Command_data list_files, Hidden_command list_manifest,
                                              Command_data list_tags]
                     }

query :: DarcsCommand
query = command_alias "query" list
\end{code}

\subsection{darcs list}

The query command provides access to several subcommands which can be
used to investigate the state of a repository.

\input{Darcs/Commands/ListFiles.lhs}
\input{Darcs/Commands/ListTags.lhs}
